#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

import PAR2Set
from RarslaveCommon import *

#
# This is a regular zip file type
#
# It will detect sets like the following:
# X.par2
# X.vol0+1.par2
# X.vol1+2.par2
# X.zip
# ABC.zip
#
# Where the PAR2 files protect a file named X.zip and/or ABC.zip.
#

def detect_ZIP (name_files, prot_files):
	all_files = no_duplicates (name_files + prot_files)
	return has_a_match ('^.*\.zip$', all_files)


class PAR2Set_ZIP (PAR2Set.PAR2Set):

	def __repr__ (self):
		return 'ZIP'

	def find_extraction_heads (self):
		return find_matches ('^.*\.zip', self.all_files)

	def extraction_function (self, file, todir):
		ZIP_CMD = self.config_get_value ('commands', 'unzip')

		cmd = ZIP_CMD % (file, todir)
		ret = run_command (cmd)

		# Check error code
		if ret != 0:
			return -EEXTRACT

		return SUCCESS

